//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on 2/25/2012.
//
#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QFileDialog>
#include <QDir>
#include <QDebug>
#include <QSettings>
#include <QWebFrame>

#include "TKLog.h"
#include "DNGlobal.h"
#include "DNAlert.h"
#include "DNEngine.h"
#include "TKConsole.h"
#include "dnwebinterface.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    mEngine(NULL), mWebInterface(NULL),
    mHasPendingConsoleMessage(false)
{
    ui->setupUi(this);

    QSettings settings("dennco project", "dennco engine");
    QVariant defaultPath(QDir::homePath());
    QString  contentPath = settings.value("content_path", defaultPath).toString();

    ui->filePath->setText(contentPath);

    ui->console->document()->setMaximumBlockCount(2000);
    QFont monofont("Courier");
    monofont.setStyleHint(QFont::Monospace);
    ui->console->setFont(monofont);
    TKLog::setDestination(this);
    connect(this, SIGNAL(consoleUpdated()), this, SLOT(update()), Qt::AutoConnection);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    stopEngine();
    deleteCurrentEngine();
    QMainWindow::closeEvent(event);
}


void MainWindow::on_chooseDirButton_clicked()
{
    if (QString::compare(ui->startButton->text(), "Stop", Qt::CaseInsensitive) == 0)
    {
        DNAlert::show("Please stop engine", "Dennco engine is running.\nPlease stop engine before switching directory.");
        return;
    }
    QString dirPath = QFileDialog::getExistingDirectory(this,"Set dennco setting file",ui->filePath->text());
    qDebug()<< dirPath;
    if (dirPath.length() > 0)
    {
        ui->filePath->setText(dirPath);

        QSettings settings("dennco project", "dennco engine");
        settings.setValue("content_path", QVariant(dirPath));
    }
}


void MainWindow::on_startButton_clicked()
{
    qDebug()<< "start button clicked";
    if (QString::compare(ui->startButton->text(), "Start", Qt::CaseInsensitive) == 0)
    {
        deleteCurrentEngine();

        if (startEngine())
        {
            ui->startButton->setText("Stop");
        }
    }
    else
    {
        stopEngine();
        ui->startButton->setText("Start");
    }
}

bool MainWindow::startEngine()
{
    bool initializationFailed = false;

    mEngine = new DNEngine(ui->filePath->text().toLocal8Bit().data());
    if (!dnGlobal()->isErrorStatusNormal())
    {
        DNAlert::show(dnGlobal()->getMessage1(), dnGlobal()->getMessage2());
        dnGlobal()->resetErrorStatus();
        initializationFailed = true;
    }
    else if (!mEngine->isValid())
    {
        DNAlert::show("DNEngine initialize error", "ERROR. DNEngine not configured properly");
        initializationFailed = true;
    }

    if (!initializationFailed)
    {
        mEngine->startEngine();
        loadUI();
    }
    return !initializationFailed;
}

bool MainWindow::stopEngine()
{
    if (mEngine)
    {
        mEngine->stopEngine();
    }
    return true;
}

void MainWindow::loadUI()
{
    QString path = "file:///";
    path.append(ui->filePath->text().append(QString::fromStdString(mEngine->getUIPath())));
    QUrl pathUrl(path);
    ui->webView->load(pathUrl);
    mWebInterface = new DNWebInterface(mEngine);
    attachWebInterface();
}

void MainWindow::attachWebInterface()
{
    if (!mWebInterface)
        return;

    QWebPage *page = ui->webView->page();
    if (page)
    {
        QWebFrame *frame = page->mainFrame();
        if (frame)
        {
            frame->addToJavaScriptWindowObject("engine", mWebInterface);
            connect(frame, SIGNAL(javaScriptWindowObjectCleared()), this, SLOT(attachWebInterface()));
        }
    }
}

void MainWindow::deleteCurrentEngine()
{
    if (mEngine)
    {
        delete mEngine;
        mEngine = NULL;
    }
    if (mWebInterface)
    {
        delete mWebInterface;
        mWebInterface = NULL;
    }
}

void MainWindow::paintEvent( QPaintEvent * event )
{
    mConsoleLock.lock();
    if (mHasPendingConsoleMessage)
    {
        ui->console->append(mPendingConsoleMessage);
        mPendingConsoleMessage.clear();
        mHasPendingConsoleMessage = false;
    }
    mConsoleLock.unlock();

    QMainWindow::paintEvent(event);
}

void MainWindow::vprintf(const char *fmt, va_list ap)
{
    QString msg = QString().vsprintf(fmt,ap);
#ifdef DEBUG
    qDebug() << msg << endl;
#endif
    mConsoleLock.lock();
    if (mHasPendingConsoleMessage)
        mPendingConsoleMessage.append("\n");
    mPendingConsoleMessage.append(msg);
    mHasPendingConsoleMessage = true;
    mConsoleLock.unlock();
    emit consoleUpdated();
}

void MainWindow::vDebugPrintf(const char *fmt, va_list ap)
{
#ifdef DEBUG
    QString msg = QString().vsprintf(fmt,ap);
    qDebug() << "DEBUG:" << msg << endl;
#endif
}


