//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on 2/22/2012.
//
#include "dnqscontainer.h"

#include "TKCell.h"
#include "TKCellCode.h"
#include "TKContainer.h"
#include "TKLog.h"
#include "DNUtils.h"
#include "versioninfo.h"
#include "dnqsbasiccell.h"
#include "dnqsinputcell.h"
#include "dnqsoutputcell.h"
#include "dnqsbasicstoragecell.h"
#include "TKUICell.h"
#include "dnqscellcode.h"
#include "DNGlobal.h"

const std::string DNQSContainer::CELLTYPE_JSBASIC = "B";
const std::string DNQSContainer::CELLTYPE_OUT = "O";
const std::string DNQSContainer::CELLTYPE_IN = "I";
const std::string DNQSContainer::CELLTYPE_BASICSTORAGE = "BS";

static QScriptValue scriptPrint(QScriptContext *context, QScriptEngine *engine)
{
    if (context->argumentCount() >= 1)
    {
        QString value = context->argument(0).toString();
        TKLog::printf("%s", value.toLocal8Bit().data());
    }
    return engine->nullValue();
}

static QScriptValue scriptFQN(QScriptContext *context, QScriptEngine *engine)
{
    if (context->argumentCount() >= 2)
    {
        const char *location = context->argument(0).toString().toUtf8().constData();
        const char *name = context->argument(1).toString().toUtf8().constData();

        std::string fqnString = getFQNString(location, name);
        return QScriptValue(fqnString.c_str());
    }
    else
    {
        return engine->nullValue();
    }
}

//static
TKContainer* TKContainer::createContainer()
{
    return new DNQSContainer;
}

DNQSContainer::DNQSContainer()
{
    mQSEngine = new QScriptEngine();
    mQSGlobalObject = mQSEngine->globalObject();

    mQSGlobalObject.setProperty("print",mQSEngine->newFunction(scriptPrint, 1));
    mQSGlobalObject.setProperty("FQN",mQSEngine->newFunction(scriptFQN, 2));

    QScriptValue global = mQSEngine->newObject();
    mQSGlobalObject.setProperty("global", global,QScriptValue::Undeletable);
    global.setProperty("container_version", QScriptValue(CONTAINER_VERSION),QScriptValue::ReadOnly|QScriptValue::Undeletable);

    mQSCellContainer = mQSEngine->newObject();
    mQSGlobalObject.setProperty("__private_DNCellContainer__",mQSCellContainer);

    mEmptyCellClass = createCellCode("_DNEMPTY",CELLTYPE_JSBASIC,"");
}

DNQSContainer::~DNQSContainer()
{
    if (mQSEngine)
        delete mQSEngine;
}
TKCell* DNQSContainer::createCellWithoutCellCodeClass(std::string theLocation, std::string theName, std::string type, std::string customScript)
{
    TKCell *cell = NULL;
    bool    noScript = false;

    if (type == CELLTYPE_JSBASIC || type.length() == 0)
    {
        cell = new DNQSBasicCell(this, theLocation, theName, false);
    }
    else if (type == CELLTYPE_IN)
    {
        cell = new DNQSInputCell(this, theLocation, theName, true);
        noScript = true;
    }
    else if (type == CELLTYPE_OUT)
    {
        cell = new DNQSOutputCell(this, theLocation, theName, true);
        noScript = true;
    }
    else if (type == CELLTYPE_BASICSTORAGE)
    {
        cell = new DNQSBasicStorageCell(this, theLocation, theName, false);
    }
    else
    {
        std::string message = std::string("Failed to construct cell '").append(theLocation).append("#").append(theName);
        message.append("'\nThe cellcode requires type '").append(type).append("' but it's not a supported type.");
        TKLog::printf("%s", message.c_str());
        if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
        {
            dnGlobal()->setMessage1("Initialization failed");
            dnGlobal()->setMessage2(message);
        }
    }

    if (cell)
    {
        std::string fqnName = getFQNString(theLocation.c_str(), theName.c_str());
        mCells.insert(TKCellMap::value_type(fqnName, cell));
        if (cell->isInterface())
        {
            mInterfaceCells.insert(TKCellMap::value_type(fqnName, cell));
        }
        if (!noScript)
        {
            cell->setCellCode(mEmptyCellClass,(const void*)customScript.c_str());
        }
        else
        {
            if (customScript.length()>0)
            {
                std::string message = std::string("Failed to construct cell '").append(theLocation).append("#").append(theName);
                message.append("'\nThe cellcode is type '").append(type).append("'. This type doesn't support custom script.");
                TKLog::printf("%s", message.c_str());
                if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
                {
                    dnGlobal()->setMessage1("Initialization failed");
                    dnGlobal()->setMessage2(message);
                }
            }
        }
    }
    else
    {
        std::string fqnName = getFQNString(theLocation.c_str(),theName.c_str());
        TKLog::printf("Failed to create a cell. %s", fqnName.c_str());
    }

    return cell;
}

TKCell* DNQSContainer::createCellWithCellCodeClass(std::string theLocation, std::string theName, TKCellCode *cellCode, std::string customScript)
{
    TKCell *cell = NULL;

    std::string type = cellCode->getCellAPIName();

    if (type == CELLTYPE_JSBASIC)
    {
        cell = new DNQSBasicCell(this, theLocation, theName, false);
    }
    else if (type == CELLTYPE_OUT)
    {
        std::string message = std::string("Failed to construct cell '").append(theLocation).append("#").append(theName);
        message.append("'\nThe cellcode type '").append(type).append("' doesn't support to have a CellCode class.");
        TKLog::printf("%s", message.c_str());
        if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
        {
            dnGlobal()->setMessage1("Initialization failed");
            dnGlobal()->setMessage2(message);
        }
    }
    else if (type == CELLTYPE_IN)
    {
        std::string message = std::string("Failed to construct cell '").append(theLocation).append("#").append(theName);
        message.append("'\nThe cellcode requires type '").append(type).append("' doesn't support to have a CellCode class.");
        TKLog::printf("%s", message.c_str());
        if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
        {
            dnGlobal()->setMessage1("Initialization failed");
            dnGlobal()->setMessage2(message);
        }
    }
    else
    {
        std::string message = std::string("Failed to construct cell '").append(theLocation).append("#").append(theName);
        message.append("'\nThe cellcode requires type '").append(type).append("' but it's not a supported type.");
        TKLog::printf("%s", message.c_str());
        if (dnGlobal()->updateErrorStatus(DNGlobal::ERROR))
        {
            dnGlobal()->setMessage1("Initialization failed");
            dnGlobal()->setMessage2(message);
        }
    }


    if (cell)
    {
        std::string fqnName = getFQNString(theLocation.c_str(), theName.c_str());
        mCells.insert(TKCellMap::value_type(fqnName, cell));
        cell->setCellCode(cellCode,(const void*)customScript.c_str());

        if (cell->isInterface())
        {
            mInterfaceCells.insert(TKCellMap::value_type(fqnName, cell));
        }
    }
    else
    {
        std::string fqnName = getFQNString(theLocation.c_str(),theName.c_str());
        TKLog::printf("Failed to create a cell. %s", fqnName.c_str());
    }

    return cell;
}

TKCellCode* DNQSContainer::createCellCode(std::string theName, std::string theAPIType, std::string code)
{
    DNQSCellCode *cellCode = new DNQSCellCode(theName, theAPIType, this, code);

    if (cellCode)
    {
        mCellCodes.insert( std::map<std::string, TKCellCode*>::value_type(theName, cellCode));
    }
    return cellCode;
}

void DNQSContainer::setValue(std::string key, float value)
{
    mQSGlobalObject.setProperty(QString::fromStdString(key), QScriptValue(value));
}

float DNQSContainer::getValue(std::string key)
{
    QScriptValue v = mQSGlobalObject.property(QString::fromStdString(key));
    if (v.isNumber())
    {
        return v.toNumber();
    }
    else
    {
        return 0;
    }
}
